window.ble = {
    debug: false,//是否输出日志
    module: null,
    initialized: false,
    connected: false,
    login: false,//已登录
    scanning: false,//正在扫描
    connecting: false,//正在连接中
    keepHeart: true,//是否保持心跳检测
    reconnectCount: 0,//重连次数
    connectCallBack: function () {

    },//因为蓝牙连接的流程比较复杂，所以把连接回调保存起来，以前作为参数传递造成内存泄漏
    uuids: {//已连接参数
        peripheralUUID: '',
        serviceUUID: '',
        writeCharUUID: '',
        notifyCharUUID: '',
    },
    agreement: {
        data: '',//拆包数据
        header: 'AABB',
        footer: 'CCDD',
        password: '',//加密秘钥
        step: '00',//步数，由下位机在第一次握手协议中告知本次通讯的起始步数
        sendTime: 0,//发送时间
        sending: false,
        waitSendArr: [],//等待发送的指令
        actionTime: 0,
    },
    dataCallback: function () {
    },//数据回执,当接收到数据时执行此函数
    /**
     * 解码
     * @param hex
     */
    decoder: function (hex) {
        var str = hex.slice(4, -8),
            crc = hex.slice(-8, -4),
            action = str.substr(0, 2),
            len = Util.hexToDec(str.substr(2, 2)) * 2,
            value = len ? str.substr(4, len) : '';
        this.agreement.step = str.slice(-2);
        if (Util.toModbusCRC16(str) !== crc) return {action: '00'};
        if (!/(13|14)/.test(action)) {
            value = Util.encrypt(this.agreement.password, str.substr(4), 1);
            this.agreement.step = value.substr(len, 2);
            value = value.slice(0, len);
        }
        return {action: action, value: value}
    },
    /**
     * 日志打印
     * @param ret
     */
    log: function (ret) {
        if (this.debug) console.log('ble:' + ret);
    },
    /**
     * 初始化
     * @returns {Promise<any>}
     */
    init: function () {
        this.module = api.require('ble');
        return new Promise(function (resolve) {
            new Promise(function (resolve, reject) {
                switch (api.systemType) {
                    case 'android':
                        ble.module.openBluetooth(function (ret) {
                            if (ret.status) resolve();
                            else reject();
                        });
                        break;
                    case 'ios':
                        ble.module.sysAuth(function (ret) {
                            if (ret.status) resolve();
                            else reject();
                        });
                        break;
                }
            }).then(function () {
                return new Promise(function (resolve, reject) {
                    ble.module.initManager({single: true}, function (ret) {
                        ble.log('初始化-' + JSON.stringify(ret));
                        switch (ret.state) {
                            case 'poweredOn':
                                ble.initialized = true;
                                ble.heart();
                                resolve();
                                break;
                            case 'unknown':
                                break;
                            case 'poweredOff':
                            case 'resetting':
                            case 'unauthorized':
                            case 'unsupported':
                                reject();
                                break;
                        }
                    })
                });
            }).then(resolve).catch(function () {
                api.confirm({msg: '请在设置中打开蓝牙权限', buttons: ['取消', '确定']}, function (ret) {
                    if (ret.buttonIndex === 1) api.closeWidget({silent: true});
                    else ble.init();
                })
            });
        });
    },
    scanBle: function () {
        if (this.initialized) {
            if (!this.scanning) {
                this.log('开始扫描蓝牙');
                this.scanning = true;
                this.module.stopScan();
                this.module.scan();
                setTimeout(function () {
                    ble.scanning=false;
                    if (ble.scanning) ble.module.stopScan();
                }, 10000)
            }
        } else {
            ble.init();
        }
    },
    getBleList: function () {
        return new Promise(function (resolve) {
            ble.module.getPeripheral(function (ret) {
                if (ret.peripherals && ret.peripherals.length){
                    resolve(ret.peripherals);
                }else{
                    resolve([]);
                }
            })
        });
        // if(conn<1){
        //     this.scanning=false;
        //     this.module.stopScan();
        //     if(this.bleList.length===0)_alert('未搜索到任何设备,请检查是否开启了定位服务')
        // }else{
        //     this.module.getPeripheral(function (ret) {
        //         if (ret.peripherals && ret.peripherals.length) {
        //             ble.bleList = ret.peripherals;
        //         }
        //         setTimeout(function () {
        //             ble.getBleList(--conn)
        //         }, 1000)
        //     })
        // }
    },
    /**
     * 连接蓝牙
     * @param uuid 待连接的UUID
     */
    connectBle: function (uuid, callback) {
        this.scanning = false;
        this.module.stopScan();
        this.connectCallBack = callback;
        this.uuids.peripheralUUID = uuid;
        this.reconnectCount = 0;//重置重连次数
        this.startConnect(uuid);
        setTimeout(function () {
            if (!ble.login) {
                ble.connectCallBack({ret: status, code: 2, msg: '连接失败'});
                ble.disconnectBle(1)
            }
        }, 10000);
    },
    /**
     * 开始连接
     * @param uuid 连接的蓝牙uuid
     * @param conn 连接的次数
     * @returns {*}
     */
    startConnect: function (uuid) {
        ble.log((this.connecting ? '已经在连接-' : '准备连接-') + this.reconnectCount + ':' + uuid);
        if (this.connecting) return;
        if (this.reconnectCount++ > 5) {
            ble.connectCallBack({status: false});
            return ble.disconnectBle(1);
        }
        this.connecting = true;
        this.module.connect({
            peripheralUUID: uuid
        }, function (ret) {
            ble.connected = ret.status;
            //如果是被硬件端断开连接,会有状态返回,如果是调用disconnect方法断开,无返回
            ble.log('蓝牙连接状态-' + ret.status + ',登录状态:' + ble.login);
            if (ret.status) {
                ble.connected = true;
                ble.getServiceUUID();
            } else if (!ble.login) {
                ble.disconnectBle().then(function (uuid) {
                    ble.startConnect(uuid);
                });
            } else {
                if (ble.agreement.sending) ble.dataCallback({action: '00', value: '蓝牙连接被断开'});
                ble.disconnectBle(1)
            }
        });
    },
    getServiceUUID: function () {
        //因为小米10机型有概率获取不到服务uuid,所以加个延时判断
        setTimeout(function () {
            if (!ble.uuids.serviceUUID) {
                ble.log('没有获取到服务,断开重连' + ble.reconnectCount);
                ble.disconnectBle().then(function (uuid) {
                    ble.startConnect(uuid)
                });
            }
        }, 1500);
        ble.module.discoverService({
            peripheralUUID: ble.uuids.peripheralUUID
        }, function (ret) {
            ble.log('服务UUID:' + JSON.stringify(ret));
            if (ret.status) {
                for (var i = 0; i < ret.services.length; i++) {
                    if (/ffe0/i.test(ret.services[i])) {
                        ble.uuids.serviceUUID = ret.services[i];
                        //获取特征码
                        ble.getCharUUID();
                    }
                }
            } else {
                ble.connectCallBack({status: false, code: 4, msg: '获取服务失败'});
                ble.disconnectBle(1);
            }
        })
    },
    getCharUUID: function () {
        ble.module.discoverCharacteristics({
            peripheralUUID: ble.uuids.peripheralUUID,
            serviceUUID: ble.uuids.serviceUUID
        }, function (ret) {
            ble.log('特征码UUID-' + JSON.stringify(ret));
            if (ret.status) {
                for (var i = 0; i < ret.characteristics.length; i++) {
                    var _char = ret.characteristics[i].uuid;
                    if (/ffe1/i.test(_char)) ble.uuids.writeCharUUID = _char;
                    else if (/ffe2/i.test(_char)) ble.uuids.notifyCharUUID = _char;
                }
                if (ble.uuids.writeCharUUID && ble.uuids.notifyCharUUID) {
                    ble.startNotify();
                    setTimeout(function () {
                        ble.handshake()
                    }, 1000)
                }
            } else {
                ble.disconnectBle();
                ble.connectCallBack({status: false, code: 5, msg: '获取特征码失败'});
            }
        })
    },
    startNotify: function () {
        this.module.setNotify({
            peripheralUUID: ble.uuids.peripheralUUID,
            serviceUUID: ble.uuids.serviceUUID,
            characteristicUUID: ble.uuids.notifyCharUUID
        }, function (ret, err) {
            if (ret.status) {
                // console.log('收到:'+ret.characteristic.value);
                var value = '';
                switch (api.systemType) {
                    case 'android':
                        value = ret.characteristic.value;
                        break;
                    case 'ios':
                        if (ret.characteristic.value.indexOf('bytes=') > -1) value = ret.characteristic.value.slice(ret.characteristic.value.indexOf('bytes=') + 8, -1);
                        else value = ret.characteristic.value;
                }
                ble.agreement.data += value.toUpperCase();
                if (/AABB\w+CCDD/.test(ble.agreement.data)) {
                    ble.agreement.sending = false;
                    var hex = ble.agreement.data.match(/AABB\w+CCDD/)[0], data = ble.decoder(hex);
                    if (data.action !== '06') ble.log('receive-' + ble.agreement.data + ',code:' + JSON.stringify(data));
                    switch (data.action) {
                        case '13'://同步计数
                            ble.agreement.step = data.value;
                            ble.sendData('14');
                            break;
                        case '14'://查询秘钥
                            ble.agreement.password = data.value.substr(0, 32);
                            ble.log('password:' + ble.agreement.password);
                            ble.connectCallBack({status: true});
                            ble.connectCallBack = function () {
                            };
                            ble.login = true;
                            ble.reconnectCount = 6;//防止断开后继续重连
                            ble.connecting = false;
                            api.setGlobalData({key: 'ble', value: ble.uuids});
                            api.setGlobalData({key: 'pass', value: ble.agreement.password});
                            break;
                        case '06'://心跳
                            break;
                        default:
                            ble.dataCallback(data)
                    }
                    ble.agreement.data = ble.agreement.data.replace(/AABB\w+CCDD/, ' ');
                    ble.agreement.data = ble.agreement.data.replace(/(\S+\s+|\s+)/, '');
                    if (ble.agreement.waitSendArr.length > 0) {
                        ble.sendHex(ble.agreement.waitSendArr.shift())
                    }
                }
            }
        });
    },
    /**
     * 心跳,为了保持与蓝牙终端的连接,每2秒检查一次连接状态和指令发送状态,
     * 如果蓝牙连接则判断是否有指令在发送,如果有则判断接收是否超过有效时间,
     * 超过则调用接收回执,回复FF;
     * 如果没有指令在发送则发送心跳指令来保持连接;
     * 然后延时2秒继续调用;
     */
    heart: function () {
        if (this.login && this.keepHeart) {
            // ble.log('检查心跳' + this.agreement.sending + ',' + api.winName);
            if (this.agreement.sending) {
                if (new Date().getTime() - this.agreement.sendTime > 3000) {
                    ble.log('返回错误指令:' + api.winName);
                    this.dataCallback({action: 'FF', value: '超时'});
                }
            } else {
                // ble.log('发送心跳' + api.winName);
                this.sendData('06');
            }
        }
        ;
        // ble.log('下次继续');
        setTimeout(function () {
            ble.heart();
        }, 2000);
    },
    //停止心跳
    stopHeart: function () {
        this.keepHeart = false;
    },
    //开启心跳
    startHeart: function () {
        this.keepHeart = true;
    },
    activeConnect() {
        var uuids = api.getGlobalData({key: 'ble'});
        if (uuids) {
            this.agreement.password = api.getGlobalData({key: 'pass'});
            // this.login=true;
            this.connected = true;
            this.uuids = uuids;
            this.startNotify();
            ble.log('蓝牙连接已激活');
            setTimeout(function () {
                ble.sendData('13', Util.decToHex(new Date().getTime(), 6));
            }, 1000);
        }
    },
    /**
     * 断开蓝牙,更新全局变量,重置蓝牙状态
     * @param flag 是否终止连接函数回执调用
     */
    disconnectBle: function (flag) {
        return new Promise(function (resolve) {
            ble.log('断开蓝牙方法-当前蓝牙是否连接:' + ble.connected + '当前页面:' + api.winName);
            var uuid = ble.connected ? ble.uuids.peripheralUUID : '';
            ble.uuids = {};
            ble.connecting = false;
            ble.connected = false;
            ble.login = false;
            ble.agreement.sending = false;
            ble.agreement.waitSendArr.length = 0;
            api.setGlobalData({key: 'ble', value: ''});
            if (flag) ble.connectCallBack = function () {};
            if (uuid) {
                ble.log('断开蓝牙');
                ble.module.disconnect({peripheralUUID: uuid}, function (ret) {
                    ble.log('执行断开蓝牙方法:' + JSON.stringify(ret));
                    resolve(uuid)
                });
            } else {
                resolve(uuid)
            }
        })
        // api.setGlobalData({key: 'ble', value: {}});
    },

    /**
     * 握手协议
     */
    handshake: function () {
        this.log('握手:');
        this.sendData('13', Util.decToHex(new Date().getTime(), 6))
    },
    sendData: function (action, data) {
        data=data||"";
        if(data.length%2>0)return this.log("数据长度不对:"+data.length+"--"+data)
        if (!this.uuids.notifyCharUUID) return false;
        var len = data ? Util.decToHex(data.length / 2, 1) : '00',
            waitD = (data ? data : '') + this.agreement.step;
        if (!/(13|14)/.test(action)) {
            if(action!=='06')ble.log('原始数据:'+waitD);
            waitD = Util.encrypt(this.agreement.password, waitD)
        }
        if (action !== '06') this.agreement.actionTime = new Date().getTime();
        var hexStr = this.agreement.header + action + len + waitD + Util.toModbusCRC16(action + len + waitD) + this.agreement.footer;
        if (this.agreement.sending) {
            //正在处于发送状态
            this.log('等待发送');
            this.agreement.waitSendArr.push(hexStr);
        } else {
            this.sendHex(hexStr)
        }
    },
    sendHex: function (hex) {
        this.log('send:' + hex);
        this.module.writeValueForCharacteristic({
            peripheralUUID: ble.uuids.peripheralUUID,
            serviceUUID: ble.uuids.serviceUUID,
            characteristicUUID: ble.uuids.writeCharUUID,
            value: hex,
            writeType: 'auto'
        });
        this.agreement.sendTime = new Date().getTime();
        this.agreement.sending = true;
    }
};